/*
 * Copyright (c) 2017 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *	copyright notice, this list of conditions and the following
 *	disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *	copyright notice, this list of conditions and the following
 *	disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 *      Event Manager
 */

#ifndef _OSM_ACT_MGR_H_
#define _OSM_ACT_MGR_H_

#include<arpa/inet.h>
#include<sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <complib/cl_dispatcher.h>
#include <iba/ib_types.h>

struct osm_sm;
struct osm_log;

#define SUBJECTS_NUM 3
#define OSM_ACTIVITY_SUBJECTS_NONE "none"
#define OSM_ACTIVITY_SUBJECTS_ALL "all"

typedef enum osm_act_mgr_mc_state {
	OSM_ACT_MGR_MC_LEAVE = -1,
	OSM_ACT_MGR_MC_JOIN = 1
} osm_act_mgr_mc_state_enum;

typedef enum osm_activity_mgr_prtn_del_reason {
	OSM_ACT_MGR_PRTN_EMPTY = 0,
	OSM_ACT_MGR_PRTN_DUPLICATE,
	OSM_ACT_MGR_PRTN_SHUTDOWN
} osm_activity_mgr_prtn_del_reason_enum;

typedef enum osm_activity_mgr_change {
	OSM_ACT_MGR_CREATE = 0,
	OSM_ACT_MGR_DELETE
} osm_activity_mgr_change_enum;

typedef enum osm_activity_mgr_state {
	OSM_ACT_MGR_STATE_DOWN = 1,
	OSM_ACT_MGR_STATE_INIT,
	OSM_ACT_MGR_STATE_ARMED, /* Not used for vports */
	OSM_ACT_MGR_STATE_ACTIVE
} osm_activity_mgr_state_enum;

typedef enum subject_id {
	MC_SUBJ_ID = 0,
	PRTN_SUBJ_ID,
	VIRT_SUBJ_ID
} subject_id_t;

/****s* EventMgr/osm_activity_mgr_t
* NAME
*  osm_activity_mgr_t
*
* DESCRIPTION
*	This type represents an event manager.
*
*	The manager responsible for handling events reported by the SM.
*
* SYNOPSIS
*/
typedef struct osm_activity_mgr {
	cl_dispatcher_t disp;
	cl_disp_reg_handle_t disp_mcm_mem_h;
	cl_disp_reg_handle_t disp_mcg_change_h;
	cl_disp_reg_handle_t disp_prtn_create_h;
	cl_disp_reg_handle_t disp_prtn_del_h;
	cl_disp_reg_handle_t disp_prtn_add_guid_h;
	cl_disp_reg_handle_t disp_prtn_del_guid_h;
	cl_disp_reg_handle_t disp_virt_discover_h;
	cl_disp_reg_handle_t disp_vport_change_h;
	struct osm_log *p_log;
	uint8_t *p_sm_state;
	pid_t pid;
	boolean_t enabled_subjects[SUBJECTS_NUM];
	FILE *file;
	boolean_t enabled;
} osm_activity_mgr_t;

/****f* EventMgr: EventMgr/osm_activity_mgr_init
* NAME osm_activity_mgr_init
*
* DESCRIPTION
*	Init event manager
*
* SYNOPSIS
*
*/
ib_api_status_t osm_activity_mgr_init(osm_activity_mgr_t *p_mgr, struct osm_sm *p_sm);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager object
*
*	p_sm
*		Pointer to sm object
* RETURN VALUE
*	Init Status
*/

/****f* EventMgr: EventMgr/enable_activity_subjects
* NAME enable_activity_subjects
*
* DESCRIPTION
*	Set the subjects to report activities for (according to configuration)
*
* SYNOPSIS
*
*/
void enable_activity_subjects(osm_activity_mgr_t *p_mgr, struct osm_sm *p_sm);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager object
*
*	p_sm
*		Pointer to sm object
*
* RETURN VALUE
*	N/A
*/

/****f* EventMgr: EventMgr/osm_activity_mgr_destroy
* NAME osm_activity_mgr_destroy
*
* DESCRIPTION
*	Destroy event manager
*
* SYNOPSIS
*
*/
void osm_activity_mgr_destroy(osm_activity_mgr_t *p_mgr);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager object
* RETURN VALUE
*	N/A
*/

/****f* EventMgr: EventMgr/osm_act_mgr_subj_enabled
* NAME osm_act_mgr_subj_enabled
*
* DESCRIPTION
*	Check if specific subject is enabled in activity manager
*
* SYNOPSIS
*
*/
static inline boolean_t osm_act_mgr_subj_enabled(osm_activity_mgr_t *p_mgr,
						 subject_id_t id)
{
	return (p_mgr->enabled && p_mgr->enabled_subjects[id]);
}

/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	id
*		subject id to check
*
* RETURN VALUE
*	Boolean indication whether requested subject is enabled
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_mcm_member
* NAME osm_act_mgr_post_mcm_member
*
* DESCRIPTION
*	Handle event of mcm member join or leave
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_mcm_member(osm_activity_mgr_t *p_mgr, ib_net16_t mlid,
				 char *mgid, ib_net64_t port_guid,
				 osm_act_mgr_mc_state_enum join_state);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	mlid
*		Mlid of group on which event occured
*
*	mgid
*		Mgid of group on which event occured
*
*	port_guid
*		Port guid of the member who caused the event
*
*	join_state
*		Join state which occured in group (join/leave)
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_mcg_change
* NAME osm_act_mgr_post_mcg_change
*
* DESCRIPTION
*	Handle event of mc group create/delete
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_mcg_change(osm_activity_mgr_t *p_mgr, ib_net16_t mlid,
				 char *mgid,
				 osm_activity_mgr_change_enum change);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	mlid
*		Mlid of group on which event occured
*
*	mgid
*		Mgid of group on which event occured
*
*	change
*		Change code: create/delete
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_prtn_create
* NAME osm_act_mgr_post_prtn_create
*
* DESCRIPTION
*	Handle event of partition creation
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_prtn_create(osm_activity_mgr_t *p_mgr,
				  ib_net16_t pkey, const char *name);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	pkey
*		Pkey of for created partition
*
*	name
*		Name of created partition
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_prtn_del
* NAME osm_act_mgr_post_prtn_del
*
* DESCRIPTION
*	Handle event of partition deletion
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_prtn_del(osm_activity_mgr_t *p_mgr, ib_net16_t pkey,
			       osm_activity_mgr_prtn_del_reason_enum del_reason);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	pkey
*		Pkey of deleted partition
*
*	del_reason
*		Reason code for partition deletion
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_prtn_add_guid
* NAME osm_act_mgr_post_prtn_add_guid
*
* DESCRIPTION
*	Handle event of adding guid to partition
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_prtn_add_guid(osm_activity_mgr_t *p_mgr, ib_net16_t pkey,
				    ib_net64_t guid, uint16_t block_index,
				         uint8_t pkey_index);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	pkey
*		Pkey value
*
*	guid
*		Guid of port which was added to partition
*
*	block_index
*		The block index in guid pkey table in which pkey added
*
*	pkey_index
*		The index within the block in which pkey added
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_prtn_del_guid
* NAME osm_act_mgr_post_prtn_del_guid
*
* DESCRIPTION
*	Handle event of deleting guid from partition
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_prtn_del_guid(osm_activity_mgr_t *p_mgr, ib_net16_t pkey,
				    ib_net64_t guid, uint16_t block_index,
				    uint8_t pkey_index);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	pkey
*		Pkey value
*
*	guid
*		Guid of port which was deleted from partition
*
*	block_index
*		The block index in guid pkey table from which pkey deleted
*
*	pkey_index
*		The index within the block from which pkey deleted
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_virt_discover
* NAME osm_act_mgr_post_virt_discover
*
* DESCRIPTION
*	Handle event of virtualization discovery
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_virt_discover(osm_activity_mgr_t *p_mgr,
				    ib_net64_t guid, ib_net16_t top_index);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	guid
*		Guid of port which its virtualization was discovered
*
*	top_index
*		The highest vport index on the port
*
* RETURN VALUE
*	None
*/

/****f* EventMgr: EventMgr/osm_act_mgr_post_vport_state_change
* NAME osm_act_mgr_post_vport_state_change
*
* DESCRIPTION
*	Handle event of vport state change
*
* SYNOPSIS
*
*/
void osm_act_mgr_post_vport_state_change(osm_activity_mgr_t *p_mgr,
					 ib_net64_t port_guid,
					 ib_net64_t vport_guid,
					 uint16_t vport_index,
					 ib_net64_t vnode_guid,
					 osm_activity_mgr_state_enum state);
/*
* PARAMETERS
*	p_mgr
*		Pointer to event manager
*
*	port_guid
*		Guid of port which the vport belong to
*
*	vport_guid
*		Guid of vport which its state changed
*
*	vport_index
*		The index of the vport on the port
*
*	vnode_guid
*		Guid of vnode to which the vport is registered
*
*	state
*		The new state of the vport
*
* RETURN VALUE
*	None
*/

#endif				/* _OSM_ACT_MGR_H_ */

